@@ -2,7 +2,7 @@
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-from logging import basicConfig, DEBUG
+from logging import DEBUG, basicConfig
 from os import getenv
 
 from .adapters import load
@@ -19,8 +19,8 @@ __credits__ = ["Tyson Smith", "Jesse Schwartzentruber"]
 # is where basicConfig should be called).
 if getenv("DEBUG"):
     basicConfig(
-        format="%(asctime)s %(levelname).1s %(name)s | %(message)s",
-        level=DEBUG)
+        format="%(asctime)s %(levelname).1s %(name)s | %(message)s", level=DEBUG
+    )
 # load Adapters
 load()
 raise SystemExit(main(GrizzlyArgs().parse_args()))
