@@ -7,29 +7,32 @@ from abc import ABCMeta, abstractmethod
 from collections import namedtuple
 from hashlib import sha1
 from json import dump, dumps, loads
-from logging import getLogger, WARNING
+from logging import WARNING, getLogger
+from os import SEEK_END, getcwd, getenv, makedirs, mkdir, scandir, stat, unlink, walk
+from os.path import basename, expanduser, isdir, isfile
+from os.path import join as pathjoin
+from os.path import realpath, relpath
 from platform import machine, system
-from os import listdir, getcwd, getenv, makedirs, mkdir, SEEK_END, stat, unlink, walk
-from os.path import basename, expanduser, isdir, isfile, join as pathjoin, realpath, relpath
-from re import compile as re_compile, DOTALL, VERBOSE
+from re import DOTALL, VERBOSE
+from re import compile as re_compile
 from shutil import copyfile, copyfileobj, move, rmtree
 from tarfile import open as tar_open
 from tempfile import mkstemp
 from time import strftime
-from zipfile import ZipFile, ZIP_DEFLATED
-
-from fasteners.process_lock import InterProcessLock
-from psutil import disk_usage
+from zipfile import ZIP_DEFLATED, ZipFile
 
 # import FuzzManager utilities
 from Collector.Collector import Collector
+from fasteners.process_lock import InterProcessLock
 from FTB.ProgramConfiguration import ProgramConfiguration
 from FTB.Signatures.CrashInfo import CrashInfo
+from psutil import disk_usage
 
 # check if boto is available for S3FuzzManager reporter
 try:
     from boto3 import resource
     from botocore.exceptions import ClientError
+
     _boto_import_error = None  # pylint: disable=invalid-name
     getLogger("botocore").setLevel(WARNING)
     getLogger("boto3").setLevel(WARNING)
@@ -40,7 +43,12 @@ except ImportError as err:
 from .stack_hasher import Stack
 from .utils import grz_tmp
 
-__all__ = ("FilesystemReporter", "FuzzManagerReporter", "Report", "S3FuzzManagerReporter")
+__all__ = (
+    "FilesystemReporter",
+    "FuzzManagerReporter",
+    "Report",
+    "S3FuzzManagerReporter",
+)
 __author__ = "Tyson Smith"
 __credits__ = ["Tyson Smith"]
 
@@ -53,32 +61,46 @@ LogMap = namedtuple("LogMap", "aux stderr stdout")
 class Report:
     DEFAULT_MAJOR = "NO_STACK"
     DEFAULT_MINOR = "0"
+    HANG_STACK_HEIGHT = 10
     MAX_LOG_SIZE = 1048576  # 1MB
 
-    __slots__ = ("_crash_info", "_logs", "_signature", "_target_binary", "path", "prefix", "stack")
+    __slots__ = (
+        "_crash_info",
+        "_logs",
+        "_signature",
+        "_target_binary",
+        "is_hang",
+        "path",
+        "prefix",
+        "stack",
+    )
 
-    def __init__(self, log_path, target_binary, size_limit=MAX_LOG_SIZE):
-        assert isinstance(log_path, str) and isdir(log_path)
+    def __init__(self, log_path, target_binary, is_hang=False, size_limit=MAX_LOG_SIZE):
+        assert isinstance(log_path, str)
         assert isinstance(target_binary, str)
         self._crash_info = None
         self._logs = self.select_logs(log_path)
         assert self._logs is not None
         self._signature = None
         self._target_binary = target_binary
+        self.is_hang = is_hang
         self.path = log_path
         # tail files in log_path if needed
         if size_limit < 1:
             LOG.warning("No limit set on report log size!")
         else:
-            for fname in listdir(log_path):
-                log_file_path = pathjoin(log_path, fname)
-                if isfile(log_file_path):
-                    Report.tail(log_file_path, size_limit)
+            with scandir(path=log_path) as contents:
+                for log in contents:
+                    if log.is_file() and log.stat().st_size > size_limit:
+                        Report.tail(log.path, size_limit)
         # look through logs one by one until we find a stack
         for log_file in (x for x in self._logs if x is not None):
             with open(log_file, "rb") as log_fp:
                 stack = Stack.from_text(log_fp.read().decode("utf-8", errors="ignore"))
             if stack.frames:
+                # limit the hash calculations to the first n frames if a hang
+                # was detected to attempt to help local bucketing
+                stack.height_limit = self.HANG_STACK_HEIGHT if is_hang else None
                 self.prefix = "%s_%s" % (stack.minor[:8], strftime("%Y-%m-%d_%H-%M-%S"))
                 self.stack = stack
                 break
@@ -138,7 +160,9 @@ class Report:
             # read in the log files and create a CrashInfo object
             if self._logs.aux is not None:
                 with open(self._logs.aux, "rb") as log_fp:
-                    aux_data = log_fp.read().decode("utf-8", errors="ignore").splitlines()
+                    aux_data = (
+                        log_fp.read().decode("utf-8", errors="ignore").splitlines()
+                    )
             else:
                 aux_data = None
             # create ProgramConfiguration that can be reported to a FM server
@@ -154,13 +178,17 @@ class Report:
                 fm_cfg = ProgramConfiguration(
                     basename(self._target_binary),
                     "x86_64" if cpu == "amd64" else cpu,
-                    system())
-            with open(self._logs.stderr, "rb") as err_fp, open(self._logs.stdout, "rb") as out_fp:
+                    system(),
+                )
+            with open(self._logs.stderr, "rb") as err_fp, open(
+                self._logs.stdout, "rb"
+            ) as out_fp:
                 self._crash_info = CrashInfo.fromRawCrashData(
                     out_fp.read().decode("utf-8", errors="ignore").splitlines(),
                     err_fp.read().decode("utf-8", errors="ignore").splitlines(),
                     fm_cfg,
-                    auxCrashData=aux_data)
+                    auxCrashData=aux_data,
+                )
         return self._crash_info
 
     @property
@@ -175,19 +203,131 @@ class Report:
         """
         if self._signature is None:
             self._signature = self.crash_info.createCrashSignature(
-                maxFrames=self.crash_signature_max_frames(self.crash_info))
+                maxFrames=self.crash_signature_max_frames(self.crash_info)
+            )
         return self._signature
 
     @staticmethod
     def crash_signature_max_frames(crash_info, suggested_frames=8):
         if set(crash_info.backtrace) & {
-                "std::panicking::rust_panic",
-                "std::panicking::rust_panic_with_hook",
+            "std::panicking::rust_panic",
+            "std::panicking::rust_panic_with_hook",
         }:
             # rust panic adds 5-6 frames of noise at the top of the stack
             suggested_frames += 6
         return suggested_frames
 
+    @staticmethod
+    def _find_ffpuppet_worker(logs):
+        """Search through list of log files for a ffpuppet worker log.
+
+        Args:
+            logs (list(str)): List of log files to search.
+
+        Returns:
+            str: The full file path if a match is found otherwise None.
+        """
+        found = None
+        for fname in (x for x in logs if "ffp_worker" in x):
+            if found is not None:
+                # we only expect one log here...
+                LOG.warning("overwriting previously selected %r", found)
+            found = fname
+        return found
+
+    @staticmethod
+    def _find_minidump(logs):
+        """Search through list of log files for a minidump log.
+
+        Args:
+            logs (list(str)): List of log files to search.
+
+        Returns:
+            str: The full file path if a match is found otherwise None.
+        """
+        re_dump_req = re_compile(
+            r"\d+\|0\|.+?\|google_breakpad::ExceptionHandler::WriteMinidump"
+        )
+        for fname in (x for x in logs if "minidump" in x):
+            with open(fname, "r") as log_fp:
+                data = log_fp.read(65536)
+                # this will select log that contains "Crash|SIGSEGV|" or
+                # the desired "DUMP_REQUESTED" log
+                # TODO: review this it may be too strict
+                # see mozilla-central/source/accessible/ipc/DocAccessibleParent.cpp#452
+                if "Crash|DUMP_REQUESTED|" not in data or re_dump_req.search(data):
+                    return fname
+        return None
+
+    @staticmethod
+    def _find_sanitizer(logs):
+        """Search through list of log files for a sanitizer (ASan, UBSan, etc...) log.
+
+        Args:
+            logs (list(str)): List of log files to search.
+
+        Returns:
+            str: The full file path if a match is found otherwise None.
+        """
+        # pattern to identify the ASan crash triggered when the parent process goes away
+        # TODO: this may no longer be required
+        re_e10s_forced = re_compile(
+            r"""
+            ==\d+==ERROR:.+?SEGV\son.+?0x[0]+\s\(.+?T2\).+?
+            #0\s+0x[0-9a-f]+\sin\s+mozilla::ipc::MessageChannel::OnChannelErrorFromLink
+            """,
+            DOTALL | VERBOSE,
+        )
+        # this is a list of Sanitizer error reports to prioritize
+        # Sanitizer reports not included below are deprioritized
+        prioritize_tokens = (
+            "use-after-",
+            "-buffer-overflow on",
+            ": SEGV on ",
+            "access-violation on ",
+            "attempting free on ",
+            "negative-size-param",
+            "-param-overlap",
+        )
+        found = None
+        for fname in (x for x in logs if "asan" in x):
+            with open(fname, "r") as log_fp:
+                data = log_fp.read(65536)
+            # look for interesting crash info in the log
+            if "==ERROR:" in data:
+                # check for e10s forced crash
+                if re_e10s_forced.search(data) is not None:
+                    continue
+                # make sure there is something that looks like a stack frame
+                if "#0 " in data:
+                    found = fname
+                    if any(x in data for x in prioritize_tokens):
+                        # this is the likely cause of the crash
+                        break
+            if found is None:
+                # UBSan error (non-ASan builds)
+                if ": runtime error: " in data:
+                    found = fname
+                # catch all (choose the one with info for now)
+                elif data:
+                    found = fname
+        return found
+
+    @staticmethod
+    def _find_valgrind(logs):
+        """Search through list of log files for a Valgrind worker log.
+
+        Args:
+            logs (list(str)): List of log files to search.
+
+        Returns:
+            str: The full file path if a match is found otherwise None.
+        """
+        for fname in (x for x in logs if "valgrind" in x):
+            if stat(fname).st_size:
+                return fname
+        return None
+
     @property
     def major(self):
         """The inclusive bucketing hash based on the stack trace
@@ -230,98 +370,33 @@ class Report:
         """
         return self._logs.aux or self._logs.stderr
 
-    @staticmethod
-    def select_logs(log_path):
-        """Scan log_path for file containing stderr, stdout and other (aux)
+    @classmethod
+    def select_logs(cls, path):
+        """Scan path for file containing stderr, stdout and other (aux)
         data and build a LogMap.
 
         Args:
-            log_path (str): Path to scan for log files.
+            path (str): Path to scan for log files.
 
         Returns:
-            LogMap: A LogMap pointing to files or None if log_path is empty.
+            LogMap: A LogMap pointing to log files or None if path is empty.
         """
-        # scan path for files
-        to_scan = list()
-        for entry in listdir(log_path):
-            full_path = pathjoin(log_path, entry)
-            if isfile(full_path):
-                to_scan.append(full_path)
+        to_scan = None
+        with scandir(path=path) as contents:
+            files = (x for x in contents if x.is_file())
+            # order by date hopefully the oldest log is the cause of the issue
+            to_scan = [x.path for x in sorted(files, key=lambda x: x.stat().st_mtime)]
         if not to_scan:
-            LOG.warning("No files found in %r", log_path)
+            LOG.warning("No files found in %r", path)
             return None
-
-        # order by creation date because the oldest log is likely the cause of the issue
-        to_scan.sort(key=lambda x: stat(x).st_mtime)
-
-        # pattern to identify the ASan crash triggered when the parent process goes away
-        # TODO: this may no longer be required
-        re_e10s_forced = re_compile(r"""
-            ==\d+==ERROR:.+?SEGV\son.+?0x[0]+\s\(.+?T2\).+?
-            #0\s+0x[0-9a-f]+\sin\s+mozilla::ipc::MessageChannel::OnChannelErrorFromLink
-            """, DOTALL | VERBOSE)
-
-        # this is a list of *San error reports to prioritize
-        # ASan reports not included below (deprioritized):
-        # stack-overflow, BUS, failed to allocate, detected memory leaks
-        interesting_sanitizer_tokens = (
-            "use-after-", "-buffer-overflow on", ": SEGV on ", "access-violation on ",
-            "negative-size-param", "attempting free on ", "-param-overlap")
-
-        log_aux = None
-        # look for sanitizer (ASan, UBSan, etc...) logs
-        for fname in (x for x in to_scan if "asan" in x):
-            # grab first chunk of log to help triage
-            with open(fname, "r") as log_fp:
-                log_data = log_fp.read(65536)
-            # look for interesting crash info in the log
-            if "==ERROR:" in log_data:
-                # check for e10s forced crash
-                if re_e10s_forced.search(log_data) is not None:
-                    continue
-                # make sure there is something that looks like a stack frame in the log
-                if "#0 " in log_data:
-                    log_aux = fname
-                    if any(x in log_data for x in interesting_sanitizer_tokens):
-                        break  # this is the likely cause of the crash
-                    continue  # probably the most interesting but lets keep looking
-            if log_aux is None:
-                # UBSan error (non-ASan builds)
-                if ": runtime error: " in log_data:
-                    log_aux = fname
-                # catch all (choose the one with info for now)
-                elif log_data:
-                    log_aux = fname
-
-        # look for Valgrind logs
+        # look for file to use as aux log
+        log_aux = cls._find_sanitizer(to_scan)
         if log_aux is None:
-            for fname in (x for x in to_scan if "valgrind" in x):
-                if stat(fname).st_size:
-                    log_aux = fname
-                    break
-
-        # prefer ASan logs over minidump logs
+            log_aux = cls._find_valgrind(to_scan)
         if log_aux is None:
-            re_dump_req = re_compile(r"\d+\|0\|.+?\|google_breakpad::ExceptionHandler::WriteMinidump")
-            for fname in (x for x in to_scan if "minidump" in x):
-                with open(fname, "r") as log_fp:
-                    log_data = log_fp.read(65536)
-                    # this will select log that contains "Crash|SIGSEGV|" or
-                    # the desired "DUMP_REQUESTED" log
-                    # TODO: review this it may be too strict
-                    # see https://searchfox.org/mozilla-central/source/accessible/ipc/DocAccessibleParent.cpp#452
-                    if "Crash|DUMP_REQUESTED|" not in log_data or re_dump_req.search(log_data):
-                        log_aux = fname
-                        break
-
-        # look for ffpuppet worker logs, worker logs should be used if nothing else is available
+            log_aux = cls._find_minidump(to_scan)
         if log_aux is None:
-            for fname in (x for x in to_scan if "ffp_worker" in x):
-                if log_aux is not None:
-                    # we only expect one log here...
-                    LOG.warning("aux log previously selected: %s, overwriting!", log_aux)
-                log_aux = fname
-
+            log_aux = cls._find_ffpuppet_worker(to_scan)
         # look for stderr and stdout log files
         log_err = None
         log_out = None
@@ -330,15 +405,12 @@ class Report:
                 log_err = fname
             elif "stdout" in fname:
                 log_out = fname
-
         result = LogMap(log_aux, log_err, log_out)
-        if not any(result):
-            LOG.warning("No logs found in %r", log_path)
-        return result
+        return result if any(result) else None
 
     @staticmethod
     def tail(in_file, size_limit):
-        """Tail the given file. This is destructive.
+        """Tail the given file. WARNING: This is destructive!
 
         Args:
             in_file (str): Path to file to work with.
@@ -348,11 +420,12 @@ class Report:
             None
         """
         assert size_limit > 0
-        if stat(in_file).st_size <= size_limit:
-            return
         with open(in_file, "rb") as in_fp:
             in_fp.seek(0, SEEK_END)
-            dump_pos = max((in_fp.tell() - size_limit), 0)
+            end = in_fp.tell()
+            if end <= size_limit:
+                return
+            dump_pos = end - size_limit
             in_fp.seek(dump_pos)
             out_fd, out_file = mkstemp(prefix="taillog_", dir=grz_tmp())
             with open(out_fd, "wb") as out_fp:
@@ -404,6 +477,7 @@ class FilesystemReporter(Reporter):
 
     def __init__(self, report_path, major_bucket=True):
         self.major_bucket = major_bucket
+        self.min_space = FilesystemReporter.DISK_SPACE_ABORT
         assert isinstance(report_path, str) and report_path
         self.report_path = report_path
 
@@ -434,8 +508,10 @@ class FilesystemReporter(Reporter):
         move(report.path, log_path)
         # avoid filling the disk
         free_space = disk_usage(log_path).free
-        if free_space < self.DISK_SPACE_ABORT:
-            raise RuntimeError("Running low on disk space (%0.1fMB)" % (free_space / 1048576.0,))
+        if free_space < self.min_space:
+            raise RuntimeError(
+                "Running low on disk space (%0.1fMB)" % (free_space / 1048576.0,)
+            )
         return dest_path
 
 
@@ -450,7 +526,7 @@ class FuzzManagerReporter(Reporter):
     QUAL_REDUCING = 4  # the testcase is currently being reduced
     QUAL_UNREDUCED = 5  # haven't attempted reduction yet (1st attempt, generic reducer)
     QUAL_REQUEST_SPECIFIC = 6  # platform specific reduction requested (2nd attempt)
-    QUAL_NO_TESTCASE = 7  # no testcase was detected (could be the "testcase" is not a testcase)
+    QUAL_NO_TESTCASE = 7  # testcase not detected ("testcase" not a testcase?)
     QUAL_REDUCER_BROKE = 8  # the testcase was reproducible, but broke during reduction
     QUAL_REDUCER_ERROR = 9  # reducer error
     QUAL_NOT_REPRODUCIBLE = 10  # could not reproduce the testcase
@@ -458,6 +534,7 @@ class FuzzManagerReporter(Reporter):
     def __init__(self, tool=None):
         self._extra_metadata = {}
         self.force_report = False
+        self.max_reports = FuzzManagerReporter.MAX_REPORTS
         self.quality = self.QUAL_UNREDUCED
         self.tool = tool  # optional tool name
 
@@ -485,15 +562,15 @@ class FuzzManagerReporter(Reporter):
 
         Arguments:
             key (str): key for this data in the metadata dict
-            value (object): JSON serializable object to be included in the FM crash metadata.
-                            The object will be deep-copied.
+            value (object): JSON serializable object to be included in the FM crash
+                            metadata. The object will be deep-copied.
 
         Returns:
             None
         """
         assert isinstance(key, str)
         assert key not in self._extra_metadata
-        # this not only does a deep copy, but also ensures that value is JSON serializable
+        # deep copy and ensure that value is JSON serializable
         self._extra_metadata[key] = loads(dumps(value))
 
     @classmethod
@@ -511,7 +588,7 @@ class FuzzManagerReporter(Reporter):
         trace_path = pathjoin(report.path, "rr-traces")
         if isdir(trace_path):
             LOG.info("Ignored rr trace")
-            self.add_extra_metadata("rr-trace",  "ignored")
+            self.add_extra_metadata("rr-trace", "ignored")
             # remove traces so they are not uploaded to FM (because they are huge)
             # use S3FuzzManagerReporter instead
             rmtree(trace_path)
@@ -522,9 +599,7 @@ class FuzzManagerReporter(Reporter):
         # that were caused by system OOM or bogus other crashes
         with open(report.preferred, "rb") as log_fp:
             log_data = log_fp.read().decode("utf-8", errors="ignore")
-        mem_errs = (
-            "ERROR: Failed to mmap",
-            ": AddressSanitizer failed to allocate")
+        mem_errs = ("ERROR: Failed to mmap", ": AddressSanitizer failed to allocate")
         # ignore sanitizer OOMs missing stack
         for msg in mem_errs:
             if msg in log_data and "#0 " not in log_data:
@@ -542,14 +617,18 @@ class FuzzManagerReporter(Reporter):
             cache_sig_file, cache_metadata = collector.search(report.crash_info)
             if cache_metadata is not None:
                 if cache_metadata["frequent"]:
-                    LOG.info("Frequent crash matched existing signature: %s",
-                             cache_metadata["shortDescription"])
+                    LOG.info(
+                        "Frequent crash matched existing signature: %s",
+                        cache_metadata["shortDescription"],
+                    )
                     if not self.force_report:
                         return None
                 elif "bug__id" in cache_metadata:
-                    LOG.info("Crash matched existing signature (bug %s): %s",
-                             cache_metadata["bug__id"],
-                             cache_metadata["shortDescription"])
+                    LOG.info(
+                        "Crash matched existing signature (bug %s): %s",
+                        cache_metadata["bug__id"],
+                        cache_metadata["shortDescription"],
+                    )
                     # we will still report this one, but no more
                     cache_metadata["frequent"] = True
                 # there is already a signature, initialize count
@@ -558,11 +637,14 @@ class FuzzManagerReporter(Reporter):
                 # there is no signature, create one locally so we can count
                 # the number of times we've seen it
                 max_frames = report.crash_signature_max_frames(report.crash_info)
-                cache_sig_file = collector.generate(report.crash_info, numFrames=max_frames)
+                cache_sig_file = collector.generate(
+                    report.crash_info, numFrames=max_frames
+                )
                 cache_metadata = {
                     "_grizzly_seen_count": 0,
                     "frequent": False,
-                    "shortDescription": report.crash_info.createShortSignature()}
+                    "shortDescription": report.crash_info.createShortSignature(),
+                }
             if cache_sig_file is None:
                 if self._ignored(report):
                     LOG.info("Report is unsupported and is in ignore list")
@@ -572,13 +654,16 @@ class FuzzManagerReporter(Reporter):
                 raise RuntimeError("Failed to create FM signature")
             # limit the number of times we report per cycle
             cache_metadata["_grizzly_seen_count"] += 1
-            if cache_metadata["_grizzly_seen_count"] >= self.MAX_REPORTS:
+            if cache_metadata["_grizzly_seen_count"] >= self.max_reports:
                 # we will still report this one, but no more
                 cache_metadata["frequent"] = True
             metadata_file = cache_sig_file.replace(".signature", ".metadata")
             with open(metadata_file, "w") as meta_fp:
                 dump(cache_metadata, meta_fp)
 
+        if report.is_hang:
+            self.add_extra_metadata("is_hang", True)
+
         # dump test cases and the contained files to working directory
         test_case_meta = []
         for test_number, test_case in enumerate(test_cases):
@@ -587,10 +672,16 @@ class FuzzManagerReporter(Reporter):
             if not isdir(dump_path):
                 mkdir(dump_path)
             test_case.dump(dump_path, include_details=True)
-        report.crash_info.configuration.addMetadata({"grizzly_input": repr(test_case_meta)})
+        report.crash_info.configuration.addMetadata(
+            {"grizzly_input": repr(test_case_meta)}
+        )
         if test_cases:
-            environ_string = " ".join("=".join(kv) for kv in test_cases[0].env_vars.items())
-            report.crash_info.configuration.addMetadata({"recorded_envvars": environ_string})
+            environ_string = " ".join(
+                "=".join(kv) for kv in test_cases[0].env_vars.items()
+            )
+            report.crash_info.configuration.addMetadata(
+                {"recorded_envvars": environ_string}
+            )
         else:
             self.quality = self.QUAL_NO_TESTCASE
         report.crash_info.configuration.addMetadata(self._extra_metadata)
@@ -612,17 +703,23 @@ class FuzzManagerReporter(Reporter):
                 for file_name in dir_files:
                     zip_fp.write(
                         pathjoin(dir_name, file_name),
-                        arcname=pathjoin(arc_path, file_name))
+                        arcname=pathjoin(arc_path, file_name),
+                    )
 
         # override tool name if specified
         if self.tool is not None:
             collector.tool = self.tool
 
         # announce shortDescription if crash is not in a bucket
-        if cache_metadata["_grizzly_seen_count"] == 1 and not cache_metadata["frequent"]:
+        if (
+            cache_metadata["_grizzly_seen_count"] == 1
+            and not cache_metadata["frequent"]
+        ):
             LOG.info("Submitting new crash %r", cache_metadata["shortDescription"])
         # submit results to the FuzzManager server
-        new_entry = collector.submit(report.crash_info, testCase=zip_name, testCaseQuality=self.quality)
+        new_entry = collector.submit(
+            report.crash_info, testCase=zip_name, testCaseQuality=self.quality
+        )
         LOG.info("Logged %d with quality %d", new_entry["id"], self.quality)
 
         # remove zipfile
@@ -656,11 +753,12 @@ class S3FuzzManagerReporter(FuzzManagerReporter):
         s3_bucket = getenv("GRZ_S3_BUCKET")
         assert s3_bucket is not None
         # check for existing minor hash in S3
-        s3 = resource("s3")
+        s3_res = resource("s3")
         s3_key = "rr-%s.tar.bz2" % (report.minor,)
         s3_url = "http://%s.s3.amazonaws.com/%s" % (s3_bucket, s3_key)
         try:
-            s3.Object(s3_bucket, s3_key).load()  # HEAD, doesn't fetch the whole object
+            # HEAD, doesn't fetch the whole object
+            s3_res.Object(s3_bucket, s3_key).load()
         except ClientError as exc:
             if exc.response["Error"]["Code"] == "404":
                 # The object does not exist.
@@ -678,7 +776,9 @@ class S3FuzzManagerReporter(FuzzManagerReporter):
 
         # Upload to S3
         rr_arc = self.compress_rr_trace(trace_path, report.path)
-        s3.meta.client.upload_file(rr_arc, s3_bucket, s3_key, ExtraArgs={"ACL": "public-read"})
+        s3_res.meta.client.upload_file(
+            rr_arc, s3_bucket, s3_key, ExtraArgs={"ACL": "public-read"}
+        )
         unlink(rr_arc)
         self.add_extra_metadata("rr-trace", s3_url)
         return s3_url
