@@ -3,14 +3,15 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
+import json
 from collections import namedtuple
 from itertools import chain
-import json
-from os import listdir, makedirs, SEEK_END, walk
-from os.path import abspath, basename, dirname, isfile, isdir, join as pathjoin, \
-    normpath, relpath
+from os import SEEK_END, listdir, makedirs, walk
+from os.path import abspath, basename, dirname, isdir, isfile
+from os.path import join as pathjoin
+from os.path import normpath, relpath
 from shutil import copyfileobj, rmtree
-from tempfile import mkdtemp, SpooledTemporaryFile
+from tempfile import SpooledTemporaryFile, mkdtemp
 from time import time
 from zipfile import BadZipfile, ZipFile
 from zlib import error as zlib_error
@@ -18,7 +19,6 @@ from zlib import error as zlib_error
 from ..target import sanitizer_opts
 from .utils import grz_tmp
 
-
 __all__ = ("TestCase", "TestFile", "TestCaseLoadFailure", "TestFileExists")
 __author__ = "Tyson Smith"
 __credits__ = ["Tyson Smith"]
@@ -29,7 +29,8 @@ class TestCaseLoadFailure(Exception):
 
 
 class TestFileExists(Exception):
-    """Raised when adding a TestFile to a TestCase that has an existing TestFile with the same name"""
+    """Raised when adding a TestFile to a TestCase that has an existing
+    TestFile with the same name"""
 
 
 TestFileMap = namedtuple("TestFileMap", "meta optional required")
@@ -37,22 +38,43 @@ TestFileMap = namedtuple("TestFileMap", "meta optional required")
 
 class TestCase:
     __slots__ = (
-        "adapter_name", "duration", "env_vars", "input_fname", "landing_page",
-        "redirect_page", "timestamp", "_existing_paths", "_files")
+        "adapter_name",
+        "duration",
+        "env_vars",
+        "hang",
+        "input_fname",
+        "landing_page",
+        "redirect_page",
+        "time_limit",
+        "timestamp",
+        "_existing_paths",
+        "_files",
+    )
 
-    def __init__(self, landing_page, redirect_page, adapter_name, input_fname=None, timestamp=None):
+    def __init__(
+        self,
+        landing_page,
+        redirect_page,
+        adapter_name,
+        input_fname=None,
+        time_limit=None,
+        timestamp=None,
+    ):
         self.adapter_name = adapter_name
         self.duration = None
         self.env_vars = dict()  # environment variables
+        self.hang = False
         self.input_fname = input_fname  # file that was used to create the test case
         self.landing_page = landing_page
         self.redirect_page = redirect_page
+        self.time_limit = time_limit
         self.timestamp = time() if timestamp is None else timestamp
         self._existing_paths = list()  # file paths in use
         self._files = TestFileMap(
             meta=list(),  # environment files such as prefs.js, etc...
             optional=list(),
-            required=list())
+            required=list(),
+        )
 
     def __enter__(self):
         return self
@@ -82,8 +104,8 @@ class TestCase:
 
         Args:
             path (str): Path to the root of the directory that contains files.
-            include_files (iterable): Paths of the files to be added to the
-                                      TestCase if they exist in path.
+            include_files (iterable(str)): Paths of the files to be added to the
+                                           TestCase if they exist in path.
             prefix (str): Path prefix to prepend to file when adding to
                           the TestCase.
 
@@ -196,11 +218,18 @@ class TestCase:
             None
 
         Returns:
-            TestCase: A copy of the TestCase instance
+            TestCase: A copy of the TestCase instance.
         """
-        result = type(self)(self.landing_page, self.redirect_page, self.adapter_name,
-                            self.input_fname, self.timestamp)
+        result = type(self)(
+            self.landing_page,
+            self.redirect_page,
+            self.adapter_name,
+            self.input_fname,
+            self.time_limit,
+            self.timestamp,
+        )
         result.duration = self.duration
+        result.hang = self.hang
         result.env_vars.update(self.env_vars)
         for entry in self._files.meta:
             result.add_meta(entry.clone())
@@ -258,9 +287,12 @@ class TestCase:
                 "adapter": self.adapter_name,
                 "duration": self.duration,
                 "env": self.env_vars,
+                "hang": self.hang,
                 "input": basename(self.input_fname) if self.input_fname else None,
                 "target": self.landing_page,
-                "timestamp": self.timestamp}
+                "time_limit": self.time_limit,
+                "timestamp": self.timestamp,
+            }
             with open(pathjoin(out_path, "test_info.json"), "w") as out_fp:
                 json.dump(info, out_fp, indent=2, sort_keys=True)
             # save meta files
@@ -276,7 +308,9 @@ class TestCase:
         Returns:
             TestFile: TestFile with matching file name otherwise None.
         """
-        for tfile in chain(self._files.meta, self._files.optional, self._files.required):
+        for tfile in chain(
+            self._files.meta, self._files.optional, self._files.required
+        ):
             if tfile.file_name == file_name:
                 return tfile
         return None
@@ -293,9 +327,9 @@ class TestCase:
                            subdirectories containing testcase data.
                         4) A single file to be used as a test case.
             load_prefs (bool): Load prefs.js file if available.
-            adjacent (str): Load adjacent files as part of the test case.
-                            This is always the case when loading a directory.
-                            WARNING: This should be used with caution!
+            adjacent (bool): Load adjacent files as part of the test case.
+                             This is always the case when loading a directory.
+                             WARNING: This should be used with caution!
 
         Returns:
             list: TestCases successfully loaded from path.
@@ -337,11 +371,13 @@ class TestCase:
         known_suppressions = ("lsan.supp", "tsan.supp", "ubsan.supp")
         for supp in listdir(path):
             if supp.lower() in known_suppressions:
-                # Update *SAN_OPTIONS environment variable to use provided suppression files.
+                # Update *SAN_OPTIONS to use provided suppression files.
                 opt_key = "%s_OPTIONS" % (supp.split(".")[0].upper(),)
                 opts = sanitizer_opts(self.env_vars.get(opt_key, ""))
                 opts["suppressions"] = "'%s'" % (pathjoin(path, supp),)
-                self.env_vars[opt_key] = ":".join("=".join((k, v)) for k, v in opts.items())
+                self.env_vars[opt_key] = ":".join(
+                    "=".join((k, v)) for k, v in opts.items()
+                )
 
     @classmethod
     def load_single(cls, path, load_prefs, adjacent=False):
@@ -372,7 +408,9 @@ class TestCase:
                 raise TestCaseLoadFailure("'test_info.json' has invalid 'target' entry")
             entry_point = basename(info["target"])
             if not isfile(pathjoin(path, entry_point)):
-                raise TestCaseLoadFailure("Entry point %r not found in '%s'" % (entry_point, path))
+                raise TestCaseLoadFailure(
+                    "Entry point %r not found in '%s'" % (entry_point, path)
+                )
             # always load all contents of a directory if a 'test_info.json' is loaded
             adjacent = True
         elif isfile(path):
@@ -382,7 +420,16 @@ class TestCase:
         else:
             raise TestCaseLoadFailure("Missing or invalid TestCase %r" % (path,))
         # create testcase and add data
-        test = cls(None, None, info.get("adapter", None), timestamp=info.get("timestamp", 0))
+        test = cls(
+            None,
+            None,
+            info.get("adapter", None),
+            input_fname=info.get("input", None),
+            time_limit=info.get("time_limit", None),
+            timestamp=info.get("timestamp", 0),
+        )
+        test.duration = info.get("duration", None)
+        test.hang = info.get("hang", False)
         if load_prefs and isfile(pathjoin(path, "prefs.js")):
             test.add_meta(TestFile.from_file(pathjoin(path, "prefs.js")))
         test.add_from_file(pathjoin(path, entry_point))
@@ -403,9 +450,8 @@ class TestCase:
                         continue
                     location = "/".join((dpath.split(path, 1)[-1], fname))
                     test.add_from_file(
-                        pathjoin(dpath, fname),
-                        file_name=location,
-                        required=False)
+                        pathjoin(dpath, fname), file_name=location, required=False
+                    )
         return test
 
     @property
@@ -425,7 +471,7 @@ class TestCase:
         """Remove optional files (by name) that are not in keep.
 
         Args:
-            keep (iterable): Filenames that will not be removed.
+            keep (iterable(str)): Files that will not be removed.
 
         Returns:
             None
@@ -486,16 +532,17 @@ class TestFile:
             file_name = file_name.lstrip("/")
         if file_name.endswith("."):
             file_name = file_name.rstrip(".")
-        if not file_name \
-                or ("/" in file_name and not file_name.rsplit("/", 1)[-1]) \
-                or file_name.startswith("../"):
+        if (
+            not file_name
+            or ("/" in file_name and not file_name.rsplit("/", 1)[-1])
+            or file_name.startswith("../")
+        ):
             raise TypeError("file_name is invalid %r" % (file_name,))
         # name including path relative to wwwroot
         self._file_name = normpath(file_name)
         self._fp = SpooledTemporaryFile(
-            dir=grz_tmp("storage"),
-            max_size=self.CACHE_LIMIT,
-            prefix="testfile_")
+            dir=grz_tmp("storage"), max_size=self.CACHE_LIMIT, prefix="testfile_"
+        )
 
     def __enter__(self):
         return self
@@ -514,7 +561,9 @@ class TestFile:
         """
         cloned = type(self)(self._file_name)
         self._fp.seek(0)
-        copyfileobj(self._fp, cloned._fp, self.XFER_BUF)  # pylint: disable=protected-access
+        copyfileobj(
+            self._fp, cloned._fp, self.XFER_BUF  # pylint: disable=protected-access
+        )
         return cloned
 
     def close(self):
@@ -601,7 +650,9 @@ class TestFile:
             file_name = basename(input_file)
         t_file = cls(file_name)
         with open(input_file, "rb") as src_fp:
-            copyfileobj(src_fp, t_file._fp, cls.XFER_BUF)  # pylint: disable=protected-access
+            copyfileobj(
+                src_fp, t_file._fp, cls.XFER_BUF  # pylint: disable=protected-access
+            )
         return t_file
 
     @property
