@@ -5,9 +5,12 @@
 
 from .utils import grz_tmp
 
+
 def test_testcase_01(mocker, tmp_path):
     """test grz_tmp()"""
-    mocker.patch("grizzly.common.utils.gettempdir", autospec=True, return_value=str(tmp_path))
+    mocker.patch(
+        "grizzly.common.utils.gettempdir", autospec=True, return_value=str(tmp_path)
+    )
     # create temp path
     path = grz_tmp()
     assert path == str(tmp_path / "grizzly")
