@@ -4,13 +4,12 @@
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 from logging import getLogger
 
-from .args import ReduceFuzzManagerIDArgs
-from .core import ReduceManager
 from ..common.fuzzmanager import Bucket, CrashEntry
 from ..common.reporter import FuzzManagerReporter
 from ..main import configure_logging
 from ..session import Session
-
+from .args import ReduceFuzzManagerIDArgs
+from .core import ReduceManager
 
 LOG = getLogger(__name__)
 
@@ -26,6 +25,7 @@ def main(args):
     """
     configure_logging(args.log_level)
     crash = CrashEntry(args.input)
+    LOG.info("Loaded crash %d", crash.crash_id)
     bucket = None
     try:
         # download the crash
@@ -50,9 +50,7 @@ def main(args):
                 Session.EXIT_ABORT: crash.testcase_quality,
                 Session.EXIT_SUCCESS: FuzzManagerReporter.QUAL_REDUCED_ORIGINAL,
                 Session.EXIT_FAILURE: FuzzManagerReporter.QUAL_NOT_REPRODUCIBLE,
-            }.get(
-                result, FuzzManagerReporter.QUAL_UNREDUCED
-            )
+            }.get(result, FuzzManagerReporter.QUAL_UNREDUCED)
             # don't ever set things back to Q4, default to Q5 for that case.
             # Q4 is only used in automation, so ABORT should never happen.
             if quality == FuzzManagerReporter.QUAL_REDUCING:
