@@ -7,23 +7,22 @@ from os import close, kill, unlink
 from os.path import abspath, isfile
 from platform import system
 from signal import SIGABRT
+
 try:
     from signal import SIGUSR1
 except ImportError:
     SIGUSR1 = None
-from time import sleep, time
 from tempfile import mkdtemp, mkstemp
-
-from psutil import AccessDenied, NoSuchProcess, Process, process_iter
+from time import sleep, time
 
 from ffpuppet import BrowserTimeoutError, FFPuppet, LaunchError
 from prefpicker import PrefPicker
+from psutil import AccessDenied, NoSuchProcess, Process, process_iter
 
-from .target_monitor import TargetMonitor
-from .target import Target, TargetLaunchError, TargetLaunchTimeout, TargetError
 from ..common.reporter import Report
 from ..common.utils import grz_tmp
-
+from .target import Target, TargetError, TargetLaunchError, TargetLaunchTimeout
+from .target_monitor import TargetMonitor
 
 __all__ = ("PuppetTarget",)
 __author__ = "Tyson Smith"
@@ -35,7 +34,9 @@ LOG = getLogger(__name__)
 class PuppetTarget(Target):
     __slots__ = ("use_rr", "use_valgrind", "_puppet", "_remove_prefs")
 
-    def __init__(self, binary, extension, launch_timeout, log_limit, memory_limit, **kwds):
+    def __init__(
+        self, binary, extension, launch_timeout, log_limit, memory_limit, **kwds
+    ):
         super().__init__(binary, extension, launch_timeout, log_limit, memory_limit)
         self.use_rr = kwds.pop("rr", False)
         self.use_valgrind = kwds.pop("valgrind", False)
@@ -44,21 +45,12 @@ class PuppetTarget(Target):
         self._puppet = FFPuppet(
             use_rr=self.use_rr,
             use_valgrind=self.use_valgrind,
-            use_xvfb=kwds.pop("xvfb", False))
+            use_xvfb=kwds.pop("xvfb", False),
+        )
         if kwds:
-            LOG.warning("PuppetTarget ignoring unsupported arguments: %s", ", ".join(kwds))
-
-    def _abort_hung_proc(self):
-        # send SIGABRT to the busiest process
-        with self._lock:
-            proc_usage = self._puppet.cpu_usage()
-        for pid, cpu in sorted(proc_usage, reverse=True, key=lambda x: x[1]):
-            LOG.debug("sending SIGABRT to pid: %r, cpu: %0.2f%%", pid, cpu)
-            try:
-                kill(pid, SIGABRT)
-            except OSError:
-                LOG.warning("Failed to send SIGABRT to pid %d", pid)
-            break
+            LOG.warning(
+                "PuppetTarget ignoring unsupported arguments: %s", ", ".join(kwds)
+            )
 
     def add_abort_token(self, token):
         self._puppet.add_abort_token(token)
@@ -85,61 +77,90 @@ class PuppetTarget(Target):
                 return False
         return True
 
+    def create_report(self, is_hang=False):
+        logs = mkdtemp(prefix="logs_", dir=grz_tmp("logs"))
+        self.save_logs(logs)
+        return Report(logs, self.binary, is_hang=is_hang)
+
     @property
     def monitor(self):
         if self._monitor is None:
+
             class _PuppetMonitor(TargetMonitor):
                 # pylint: disable=no-self-argument,protected-access
                 def clone_log(_, log_id, offset=0):
                     return self._puppet.clone_log(log_id, offset=offset)
+
                 def is_running(_):
                     return self._puppet.is_running()
+
                 def is_healthy(_):
                     return self._puppet.is_healthy()
+
                 @property
                 def launches(_):
                     return self._puppet.launches
+
                 def log_length(_, log_id):
                     return self._puppet.log_length(log_id)
+
             self._monitor = _PuppetMonitor()
         return self._monitor
 
-    def detect_failure(self, ignored, was_timeout):
+    def detect_failure(self, ignored):
         status = self.RESULT_NONE
-        is_healthy = self._puppet.is_healthy()
-        # check if there has been a crash, hang, etc...
-        if not is_healthy or was_timeout:
-            if self._puppet.is_running():
-                LOG.debug("terminating browser...")
-                if was_timeout and "timeout" not in ignored and system() == "Linux":
-                    self._abort_hung_proc()
-                    # give the process a moment to start dump
-                    self._puppet.wait(timeout=1)
+        # check if there has been a crash, hangs will appear as SIGABRT
+        if not self._puppet.is_healthy():
             self.close()
-        # if something has happened figure out what
-        if not is_healthy:
+            # something has happened figure out what
             if self._puppet.reason == FFPuppet.RC_CLOSED:
                 LOG.debug("target.close() was called")
             elif self._puppet.reason == FFPuppet.RC_EXITED:
                 LOG.debug("target closed itself")
-            elif (self._puppet.reason == FFPuppet.RC_WORKER
-                  and "memory" in ignored
-                  and "ffp_worker_memory_usage" in self._puppet.available_logs()):
+            elif (
+                self._puppet.reason == FFPuppet.RC_WORKER
+                and "memory" in ignored
+                and "ffp_worker_memory_usage" in self._puppet.available_logs()
+            ):
                 status = self.RESULT_IGNORED
                 LOG.debug("memory limit exceeded")
-            elif (self._puppet.reason == FFPuppet.RC_WORKER
-                  and "log-limit" in ignored
-                  and "ffp_worker_log_size" in self._puppet.available_logs()):
+            elif (
+                self._puppet.reason == FFPuppet.RC_WORKER
+                and "log-limit" in ignored
+                and "ffp_worker_log_size" in self._puppet.available_logs()
+            ):
                 status = self.RESULT_IGNORED
                 LOG.debug("log size limit exceeded")
             else:
+                # crash or hang (forced SIGABRT) has been detected
                 LOG.debug("failure detected, ffpuppet reason %r", self._puppet.reason)
                 status = self.RESULT_FAILURE
-        elif was_timeout:
-            LOG.debug("timeout detected")
-            status = self.RESULT_IGNORED if "timeout" in ignored else self.RESULT_FAILURE
         return status
 
+    def handle_hang(self, ignore_idle=True):
+        was_idle = False
+        if self._puppet.is_healthy():
+            proc_usage = sorted(self._puppet.cpu_usage(), key=lambda x: x[1])
+            if proc_usage:
+                pid, cpu = proc_usage.pop()
+                if ignore_idle and cpu < 15:
+                    # don't send SIGABRT if process is idle
+                    LOG.debug("ignoring idle hang (%0.1f%%)", cpu)
+                    was_idle = True
+                elif system() == "Linux":
+                    # sending SIGABRT is only supported on Linux for now
+                    # TODO: add/test on other OSs
+                    LOG.debug("sending SIGABRT to %r (%0.1f%%)", pid, cpu)
+                    try:
+                        kill(pid, SIGABRT)
+                    except OSError:
+                        LOG.warning("Failed to send SIGABRT to pid %d", pid)
+                    self._puppet.wait(timeout=10)
+        # always call close() since this function should only/always
+        # be called when there has been a timeout
+        self.close()
+        return was_idle
+
     def dump_coverage(self, timeout=15):
         assert SIGUSR1 is not None
         pid = self._puppet.get_pid()
@@ -188,7 +209,9 @@ class PuppetTarget(Target):
                     break
                 if elapsed >= timeout:
                     # timeout failure
-                    LOG.warning("gcda file open by pid %d after %0.2fs", gcda_open, elapsed)
+                    LOG.warning(
+                        "gcda file open by pid %d after %0.2fs", gcda_open, elapsed
+                    )
                     try:
                         kill(gcda_open, SIGABRT)
                     except OSError:
@@ -223,15 +246,14 @@ class PuppetTarget(Target):
                 memory_limit=self.memory_limit,
                 prefs_js=self.prefs,
                 extension=self.extension,
-                env_mod=env_mod)
+                env_mod=env_mod,
+            )
         except LaunchError as exc:
             LOG.error("FFPuppet LaunchError: %s", str(exc))
             self.close()
             if isinstance(exc, BrowserTimeoutError):
                 raise TargetLaunchTimeout(str(exc)) from None
-            log_path = mkdtemp(prefix="launch_fail_", dir=grz_tmp("logs"))
-            self.save_logs(log_path)
-            raise TargetLaunchError(str(exc), Report(log_path, self.binary)) from None
+            raise TargetLaunchError(str(exc), self.create_report()) from None
 
     def log_size(self):
         return self._puppet.log_length("stderr") + self._puppet.log_length("stdout")
@@ -243,7 +265,9 @@ class PuppetTarget(Target):
             for prefs_template in PrefPicker.templates():
                 if prefs_template.endswith("browser-fuzzing.yml"):
                     LOG.debug("using prefpicker template %r", prefs_template)
-                    tmp_fd, self._prefs = mkstemp(prefix="prefs_", suffix=".js", dir=grz_tmp())
+                    tmp_fd, self._prefs = mkstemp(
+                        prefix="prefs_", suffix=".js", dir=grz_tmp()
+                    )
                     close(tmp_fd)
                     PrefPicker.load_template(prefs_template).create_prefsjs(self._prefs)
                     LOG.debug("generated prefs.js %r", self._prefs)
