@@ -8,7 +8,6 @@ from os.path import isfile
 from re import split as resplit
 from threading import Lock
 
-
 __all__ = ("Target", "sanitizer_opts")
 __author__ = "Tyson Smith"
 __credits__ = ["Tyson Smith", "Jesse Schwartzentruber"]
@@ -42,6 +41,7 @@ class TargetError(Exception):
 
 class TargetLaunchError(TargetError):
     """Raised if a failure during launch occurs"""
+
     def __init__(self, message, report):
         super().__init__(message)
         self.report = report
@@ -57,8 +57,15 @@ class Target(metaclass=ABCMeta):
     RESULT_IGNORED = 2
 
     __slots__ = (
-        "_lock", "_monitor", "_prefs", "binary", "extension", "launch_timeout",
-        "log_limit", "memory_limit")
+        "_lock",
+        "_monitor",
+        "_prefs",
+        "binary",
+        "extension",
+        "launch_timeout",
+        "log_limit",
+        "memory_limit",
+    )
 
     def __init__(self, binary, extension, launch_timeout, log_limit, memory_limit):
         assert log_limit >= 0
@@ -79,7 +86,7 @@ class Target(metaclass=ABCMeta):
     def __exit__(self, *exc):
         self.cleanup()
 
-    def add_abort_token(self, token):  # pylint: disable=no-self-use,unused-argument
+    def add_abort_token(self, _token):  # pylint: disable=no-self-use
         LOG.warning("add_abort_token() not implemented!")
 
     @abstractmethod
@@ -94,22 +101,28 @@ class Target(metaclass=ABCMeta):
     def closed(self):
         pass
 
-    # TODO: add collect_report()?
-
     @abstractmethod
-    def detect_failure(self, ignored, was_timeout):
+    def create_report(self, is_hang=False):
         pass
 
-    def dump_coverage(self):  # pylint: disable=no-self-use
+    @abstractmethod
+    def detect_failure(self, ignored):
+        pass
+
+    def dump_coverage(self, _timeout=0):  # pylint: disable=no-self-use
         LOG.warning("dump_coverage() is not supported!")
 
+    @abstractmethod
+    def handle_hang(self, ignore_idle=True):
+        pass
+
     # TODO: move to monitor?
-    def is_idle(self, threshold):  # pylint: disable=no-self-use,unused-argument
+    def is_idle(self, _threshold):  # pylint: disable=no-self-use
         LOG.debug("Target.is_idle() not implemented! returning False")
         return False
 
     @abstractmethod
-    def launch(self):
+    def launch(self, _location, _env_mod=None):
         pass
 
     def log_size(self):  # pylint: disable=no-self-use
