@@ -3,8 +3,11 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 """test Grizzly main"""
+from pytest import mark
+
 from sapphire import Sapphire
-from .common import Adapter, Report
+
+from .common import Adapter
 from .main import main
 from .session import Session
 from .target import Target, TargetLaunchError
@@ -15,8 +18,9 @@ class FakeArgs:
         self.binary = None
         self.input = None
         self.adapter = None
-        self.cache = 0
+        self.collect = 1
         self.coverage = False
+        self.enable_profiling = False
         self.extension = None
         self.fuzzmanager = False
         self.ignore = list()
@@ -27,24 +31,29 @@ class FakeArgs:
         self.memory = 0
         self.platform = "fake-target"
         self.prefs = None
-        self.rr = False
+        self.rr = False  # pylint: disable=invalid-name
         self.relaunch = 1000
         self.s3_fuzzmanager = False
-        self.timeout = 60
+        self.time_limit = None
+        self.timeout = None
         self.tool = None
         self.valgrind = False
         self.verbose = False
         self.xvfb = False
 
+
 # TODO: these could use call_count checks
 
+
 def test_main_01(mocker):
     """test main()"""
     fake_adapter = mocker.Mock(spec=Adapter)
     fake_adapter.NAME = "fake"
-    fake_adapter.TEST_DURATION = 10
+    fake_adapter.TIME_LIMIT = 10
     mocker.patch("grizzly.main.get_adapter", return_value=lambda: fake_adapter)
-    mocker.patch.dict("grizzly.target.TARGETS", values={"fake-target": mocker.Mock(spec=Target)})
+    mocker.patch.dict(
+        "grizzly.target.TARGETS", values={"fake-target": mocker.Mock(spec=Target)}
+    )
     fake_session = mocker.patch("grizzly.main.Session", autospec=True)
     fake_session.return_value.server = mocker.Mock(spec=Sapphire)
     fake_session.EXIT_SUCCESS = Session.EXIT_SUCCESS
@@ -55,10 +64,9 @@ def test_main_01(mocker):
     args.prefs = "fake"
     args.valgrind = True
     args.xvfb = True
-    # successful run (with coverage, short timeout)
+    # successful run (with coverage)
     fake_adapter.RELAUNCH = 10
     args.coverage = True
-    args.timeout = 1
     assert main(args) == Session.EXIT_SUCCESS
     assert fake_session.mock_calls[0][-1]["coverage"]
     assert fake_session.mock_calls[0][-1]["relaunch"] == 10
@@ -66,7 +74,6 @@ def test_main_01(mocker):
     # successful run (without coverage)
     fake_adapter.RELAUNCH = 1
     args.coverage = False
-    args.timeout = 60
     assert main(args) == Session.EXIT_SUCCESS
     assert not fake_session.mock_calls[0][-1]["coverage"]
     assert fake_session.mock_calls[0][-1]["relaunch"] == 1
@@ -92,17 +99,20 @@ def test_main_01(mocker):
     args.s3_fuzzmanager = True
     assert main(args) == Session.EXIT_SUCCESS
 
-def test_main_02(mocker, tmp_path):
+
+def test_main_02(mocker):
     """test main() exit codes"""
     fake_adapter = mocker.Mock(spec=Adapter)
-    fake_adapter.TEST_DURATION = 10
+    fake_adapter.TIME_LIMIT = 10
     fake_adapter.RELAUNCH = 0
     mocker.patch("grizzly.main.get_adapter", return_value=lambda: fake_adapter)
-    mocker.patch.dict("grizzly.target.TARGETS", values={"fake-target": mocker.Mock(spec=Target)})
+    mocker.patch.dict(
+        "grizzly.target.TARGETS", values={"fake-target": mocker.Mock(spec=Target)}
+    )
     fake_session = mocker.patch("grizzly.main.Session", autospec=True)
     fake_session.EXIT_SUCCESS = Session.EXIT_SUCCESS
     fake_session.EXIT_ABORT = Session.EXIT_ABORT
-    fake_session.EXIT_ARGS = Session.EXIT_ARGS
+    fake_session.EXIT_ARGS = fake_session.EXIT_ARGS = Session.EXIT_ARGS
     fake_session.EXIT_LAUNCH_FAILURE = Session.EXIT_LAUNCH_FAILURE
     fake_session.return_value.server = mocker.Mock(spec=Sapphire)
     args = FakeArgs()
@@ -111,13 +121,41 @@ def test_main_02(mocker, tmp_path):
     fake_session.return_value.run.side_effect = KeyboardInterrupt
     assert main(args) == Session.EXIT_ABORT
     # test TargetLaunchError
-    fake_tmp = (tmp_path / "grz_tmp")
-    fake_tmp.mkdir()
-    mocker.patch("grizzly.main.grz_tmp", return_value=str(fake_tmp))
-    fake_logs = (tmp_path / "report")
-    report = mocker.Mock(spec=Report, prefix="fake_report", path=str(fake_logs))
-    fake_logs.mkdir()
-    fake_session.return_value.run.side_effect = TargetLaunchError("test", report)
+    fake_session.return_value.run.side_effect = TargetLaunchError("test", None)
     assert main(args) == Session.EXIT_LAUNCH_FAILURE
-    assert any(fake_tmp.glob("fake_report_logs"))
-    assert not fake_logs.is_dir()
+
+
+@mark.parametrize(
+    "arg_testlimit, arg_timeout, result",
+    [
+        # use default test time limit and timeout values
+        (None, None, Session.EXIT_SUCCESS),
+        # set test time limit
+        (10, None, Session.EXIT_SUCCESS),
+        # set both test time limit and timeout to the same value
+        (10, 10, Session.EXIT_SUCCESS),
+        # set timeout greater than test time limit
+        (10, 11, Session.EXIT_SUCCESS),
+        # set test time limit greater than timeout
+        (11, 10, Session.EXIT_ARGS),
+    ],
+)
+def test_main_03(mocker, arg_testlimit, arg_timeout, result):
+    """test main() time-limit and timeout"""
+    fake_adapter = mocker.Mock(spec=Adapter)
+    fake_adapter.NAME = "fake"
+    fake_adapter.RELAUNCH = 1
+    fake_adapter.TIME_LIMIT = 10
+    mocker.patch("grizzly.main.get_adapter", return_value=lambda: fake_adapter)
+    mocker.patch.dict(
+        "grizzly.target.TARGETS", values={"fake-target": mocker.Mock(spec=Target)}
+    )
+    fake_session = mocker.patch("grizzly.main.Session", autospec=True)
+    fake_session.return_value.server = mocker.Mock(spec=Sapphire)
+    fake_session.EXIT_ARGS = Session.EXIT_ARGS
+    fake_session.EXIT_SUCCESS = Session.EXIT_SUCCESS
+    args = FakeArgs()
+    args.adapter = "fake"
+    args.time_limit = arg_testlimit
+    args.timeout = arg_timeout
+    assert main(args) == result
