@@ -10,6 +10,13 @@ from .core import Sapphire
 from .job import SERVED_ALL, SERVED_NONE, SERVED_REQUEST, SERVED_TIMEOUT
 from .server_map import ServerMap
 
-__all__ = ("Sapphire", "SERVED_ALL", "SERVED_NONE", "SERVED_REQUEST", "SERVED_TIMEOUT", "ServerMap")
+__all__ = (
+    "Sapphire",
+    "SERVED_ALL",
+    "SERVED_NONE",
+    "SERVED_REQUEST",
+    "SERVED_TIMEOUT",
+    "ServerMap",
+)
 __author__ = "Tyson Smith"
 __credits__ = ["Tyson Smith"]
